package cn.com.qmth.examcloud.core.oe.admin.service.impl;

import cn.com.qmth.examcloud.commons.util.MathUtils;
import cn.com.qmth.examcloud.core.basic.api.CourseCloudService;
import cn.com.qmth.examcloud.core.basic.api.OrgCloudService;
import cn.com.qmth.examcloud.core.basic.api.bean.OrgBean;
import cn.com.qmth.examcloud.core.basic.api.request.GetCoursesByIdListReq;
import cn.com.qmth.examcloud.core.basic.api.request.GetOrgMapsReq;
import cn.com.qmth.examcloud.core.basic.api.response.GetCoursesByIdListResp;
import cn.com.qmth.examcloud.core.oe.admin.dao.ExamStatisticRepo;
import cn.com.qmth.examcloud.core.oe.admin.dao.entity.ExamStatisticEntity;
import cn.com.qmth.examcloud.core.oe.admin.service.ExamStatisticService;
import cn.com.qmth.examcloud.core.oe.admin.service.bean.statistic.ExamStatisticInfo;
import cn.com.qmth.examcloud.core.oe.admin.service.bean.statistic.OverviewInfo;
import com.google.common.collect.Lists;
import org.apache.commons.collections4.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Service
public class ExamStatisticServiceImpl implements ExamStatisticService {

    private static final Logger log = LoggerFactory.getLogger(ExamStatisticServiceImpl.class);

    @Autowired
    private ExamStatisticRepo examStatisticRepo;

    @Autowired
    private CourseCloudService courseCloudService;

    @Autowired
    private OrgCloudService orgCloudService;

    @Override
    public OverviewInfo overview(Long examId, Long courseId) {
        OverviewInfo result = new OverviewInfo();

        ExamStatisticInfo examResult = new ExamStatisticInfo();
        ExamStatisticInfo courseResult = new ExamStatisticInfo();
        result.setExamResult(examResult);
        result.setCourseResult(courseResult);

        List<ExamStatisticEntity> list = examStatisticRepo.findByExamId(examId);
        if (CollectionUtils.isEmpty(list)) {
            return result;
        }

        GetCoursesByIdListReq req = new GetCoursesByIdListReq();
        req.setCourseIdList(Lists.newArrayList(courseId));
        GetCoursesByIdListResp resp = courseCloudService.getCoursesByIdList(req);
        courseResult.setCourseName(resp.getCourseList().get(0).getName());

        for (ExamStatisticEntity v : list) {
            examResult.setAllCount(examResult.getAllCount() + v.getAllCount());
            examResult.setFinishCount(examResult.getFinishCount() + v.getFinishCount());
            examResult.setPassScoreCount(examResult.getPassScoreCount() + v.getPassScoreCount());
            examResult.setGoodScoreCount(examResult.getGoodScoreCount() + v.getGoodScoreCount());
            if (v.getCourseId().equals(courseId)) {
                courseResult.setAllCount(courseResult.getAllCount() + v.getAllCount());
                courseResult.setFinishCount(courseResult.getFinishCount() + v.getFinishCount());
                courseResult.setPassScoreCount(courseResult.getPassScoreCount() + v.getPassScoreCount());
                courseResult.setGoodScoreCount(courseResult.getGoodScoreCount() + v.getGoodScoreCount());
            }
        }

        examResult.setUnFinishCount(examResult.getAllCount() - examResult.getFinishCount());
        if (examResult.getAllCount() > 0) {
            examResult.setUnFinishRate(MathUtils.percentage(examResult.getUnFinishCount(), examResult.getAllCount()));
            examResult.setPassScoreRate(MathUtils.percentage(examResult.getPassScoreCount(), examResult.getAllCount()));
            examResult.setGoodScoreRate(MathUtils.percentage(examResult.getGoodScoreCount(), examResult.getAllCount()));
        }

        courseResult.setUnFinishCount(courseResult.getAllCount() - courseResult.getFinishCount());
        if (courseResult.getAllCount() > 0) {
            courseResult.setUnFinishRate(MathUtils.percentage(courseResult.getUnFinishCount(), courseResult.getAllCount()));
            courseResult.setPassScoreRate(MathUtils.percentage(courseResult.getPassScoreCount(), courseResult.getAllCount()));
            courseResult.setGoodScoreRate(MathUtils.percentage(courseResult.getGoodScoreCount(), courseResult.getAllCount()));
        }

        return result;
    }

    @Override
    public List<ExamStatisticInfo> overviewForOrg(Long examId, Long courseId) {
        List<ExamStatisticEntity> list = examStatisticRepo.findByExamIdAndCourseId(examId, courseId);
        if (CollectionUtils.isEmpty(list)) {
            return new ArrayList<>();
        }

        List<ExamStatisticInfo> result = new ArrayList<>();

        Map<Long, List<ExamStatisticEntity>> orgStatisticMaps = list.stream().collect(Collectors.groupingBy(ExamStatisticEntity::getOrgId));

        // 获取机构信息列表
        GetOrgMapsReq orgMapsReq = new GetOrgMapsReq();
        orgMapsReq.setOrgIds(orgStatisticMaps.keySet());
        Map<Long, OrgBean> orgMaps = orgCloudService.getOrgMaps(orgMapsReq).getOrgMaps();

        for (Map.Entry<Long, List<ExamStatisticEntity>> e : orgStatisticMaps.entrySet()) {
            ExamStatisticInfo info = new ExamStatisticInfo();
            info.setOrgId(e.getKey());

            OrgBean org = orgMaps.get(e.getKey());
            if (org != null) {
                info.setOrgName(org.getName());
            }

            for (ExamStatisticEntity v : e.getValue()) {
                info.setAllCount(info.getAllCount() + v.getAllCount());
                info.setFinishCount(info.getFinishCount() + v.getFinishCount());
                info.setPassScoreCount(info.getPassScoreCount() + v.getPassScoreCount());
                info.setGoodScoreCount(info.getGoodScoreCount() + v.getGoodScoreCount());
            }

            info.setUnFinishCount(info.getAllCount() - info.getFinishCount());
            if (info.getAllCount() > 0) {
                info.setUnFinishRate(MathUtils.percentage(info.getUnFinishCount(), info.getAllCount()));
                info.setPassScoreRate(MathUtils.percentage(info.getPassScoreCount(), info.getAllCount()));
                info.setGoodScoreRate(MathUtils.percentage(info.getGoodScoreCount(), info.getAllCount()));
            }

            result.add(info);
        }

        return result;
    }

}
